'use client';

import React, { useEffect, useState, FormEvent } from 'react';
import { AddressElement, Elements } from '@stripe/react-stripe-js';
import {
  Appearance,
  StripeAddressElementOptions,
  StripeElementsOptions,
  loadStripe,
} from '@stripe/stripe-js';
import {
  PaymentElement,
  useStripe,
  useElements,
} from '@stripe/react-stripe-js';
import { RadioGroup } from '@headlessui/react';

const stripe = loadStripe(process.env.NEXT_PUBLIC_STRIPE_PUBLISHABLE_KEY || '');

const addressOptions: StripeAddressElementOptions = {
  mode: 'billing',
  allowedCountries: ['US'],
  fields: {
    phone: 'always',
  },
  validation: {
    phone: {
      required: 'always',
    },
  },
  defaultValues: {
    address: {
      state: 'WA',
      country: 'US',
    },
  },
};

const donationLevels = [10, 50, 100, -1];

function classNames(...classes: string[]) {
  return classes.filter(Boolean).join(' ');
}

function Spinner(): React.JSX.Element {
  return (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      width="24"
      height="24"
      viewBox="0 0 24 24"
    >
      <path
        fill="currentColor"
        d="M10.72,19.9a8,8,0,0,1-6.5-9.79A7.77,7.77,0,0,1,10.4,4.16a8,8,0,0,1,9.49,6.52A1.54,1.54,0,0,0,21.38,12h.13a1.37,1.37,0,0,0,1.38-1.54,11,11,0,1,0-12.7,12.39A1.54,1.54,0,0,0,12,21.34h0A1.47,1.47,0,0,0,10.72,19.9Z"
      >
        <animateTransform
          attributeName="transform"
          dur="1s"
          repeatCount="indefinite"
          type="rotate"
          values="0 12 12;360 12 12"
        />
      </path>
    </svg>
  );
}

function CheckoutForm({
  paymentIntentID,
}: {
  paymentIntentID: string;
}): React.JSX.Element {
  const [selectedDonation, setSelectedDonation] = useState<number>(10);
  const [customAmount, setCustomAmount] = useState('');
  const [email, setEmail] = useState('');
  const [totalAmount, setTotalAmount] = useState(300);
  const [message, setMessage] = useState<string>('');
  const [isLoading, setIsLoading] = useState(false);
  const stripe = useStripe();
  const elements = useElements();

  useEffect(() => {
    if (!stripe) {
      return;
    }

    //Grab the client secret from url params
    const clientSecret = new URLSearchParams(window.location.search).get(
      'payment_intent_client_secret',
    );

    if (!clientSecret) {
      return;
    }

    stripe.retrievePaymentIntent(clientSecret).then(({ paymentIntent }) => {
      switch (paymentIntent?.status) {
        case 'succeeded':
          setMessage('Payment succeeded!');
          break;
        case 'processing':
          setMessage('Your payment is processing.');
          break;
        case 'requires_payment_method':
          setMessage('Your payment was not successful, please try again.');
          break;
        default:
          setMessage('Something went wrong.');
          break;
      }
    });
  }, [stripe]);

  useEffect(() => {
    let subtotal = 0;

    if (selectedDonation && selectedDonation !== -1) {
      subtotal += selectedDonation;
    } else if (customAmount !== '') {
      try {
        subtotal += parseFloat(customAmount);
      } catch {
        console.error('');
      }
    }

    setTotalAmount(subtotal);

    fetch('api/stripe_intent', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify({
        amount: subtotal * 100,
        payment_intent_id: paymentIntentID,
      }),
    });
  }, [paymentIntentID, selectedDonation, customAmount, email]);

  const handleSubmit = async (e: FormEvent) => {
    e.preventDefault();

    if (!stripe || !elements) {
      // Stripe.js has not yet loaded.
      return;
    }

    setIsLoading(true);

    const { error } = await stripe.confirmPayment({
      elements,
      confirmParams: {
        return_url: window.location.origin + '/donate/thank-you',
        receipt_email: email,
        payment_method_data: {
          billing_details: {
            // Other details are filled automatically by address form.
            email,
          },
        },
      },
    });

    if (error.type === 'card_error' || error.type === 'validation_error') {
      setMessage(error.message || '');
    } else {
      setMessage('An unexpected error occured.');
    }

    setIsLoading(false);
  };

  return (
    <>
      <form
        id="payment-form"
        onSubmit={handleSubmit}
        className="m-auto space-y-4"
      >
        {/* Amount */}
        <RadioGroup
          value={selectedDonation}
          onChange={setSelectedDonation}
          className="space-y-3"
        >
          <RadioGroup.Label className="text-base font-semibold leading-6 text-gray-900">
            Donation amount
          </RadioGroup.Label>
          <div className="grid grid-cols-3 gap-3 sm:grid-cols-5">
            {donationLevels.map((option) => (
              <RadioGroup.Option
                key={option}
                value={option}
                className={({ active, checked }) =>
                  classNames(
                    'cursor-pointer focus:outline-none',
                    option === -1 ? 'col-span-2' : '',
                    checked
                      ? 'ring-2 ring-indigo-600'
                      : 'text-gray-900 ring-1 ring-inset ring-gray-200 hover:bg-gray-50',
                    'flex items-center justify-center rounded-md bg-white px-3 py-3 text-sm font-semibold sm:flex-1',
                  )
                }
              >
                {option === -1 ? (
                  <RadioGroup.Label
                    as="span"
                    className="flex flex-col items-center"
                  >
                    <div className="relative rounded-md shadow-sm">
                      <div className="pointer-events-none absolute inset-y-0 left-0 flex items-center pl-3">
                        <span className="text-gray-500 sm:text-sm">$</span>
                      </div>
                      <input
                        type="number"
                        name="custom-amount"
                        id="custom-amount"
                        value={customAmount}
                        min="0"
                        step="1"
                        onChange={(e) => setCustomAmount(e.target.value)}
                        className="block w-full rounded-md border-0 py-1.5 pl-7 pr-2 text-gray-900 ring-1 ring-inset ring-gray-200 placeholder:text-gray-400 focus:ring-1 focus:ring-inset focus:ring-gray-500 sm:text-sm sm:leading-6"
                        placeholder="Custom"
                        aria-describedby="price-currency"
                      />
                    </div>
                  </RadioGroup.Label>
                ) : (
                  <RadioGroup.Label as="span">
                    <span>${option}</span>
                  </RadioGroup.Label>
                )}
              </RadioGroup.Option>
            ))}
          </div>
        </RadioGroup>

        <div className="space-y-3">
          <h2 className="text-base font-semibold leading-6 text-gray-900">
            About you
          </h2>

          <div className="mb-3 rounded-md px-3 pb-1.5 pt-2.5 shadow-sm ring-1 ring-inset ring-gray-200 focus-within:ring-2 focus-within:ring-indigo-600">
            <label
              htmlFor="email"
              className="block text-xs font-medium text-gray-900"
            >
              Email
            </label>
            <input
              type="email"
              name="email"
              id="email"
              value={email}
              onChange={(e) => setEmail(e.target.value)}
              className="block w-full border-0 p-0 text-gray-900 placeholder:text-gray-400 focus:ring-0 sm:text-sm sm:leading-6"
              placeholder="you@example.com"
            />
          </div>
          <AddressElement options={addressOptions} />
        </div>

        <div className="space-y-3">
          <h2 className="text-base font-semibold leading-6 text-gray-900">
            Payment
          </h2>
          <PaymentElement id="payment-element" />
        </div>

        <button
          className="mt-6 flex w-full justify-center rounded-md border border-transparent bg-indigo-600 px-4 py-2 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2"
          disabled={isLoading || !stripe || !elements}
          id="submit"
        >
          {isLoading ? <Spinner /> : 'Pay $' + totalAmount}
        </button>

        {/* Show any error or success messages */}
        {message && (
          <div className="text-red-500" id="payment-message">
            {message}
          </div>
        )}
      </form>
    </>
  );
}

const DEFAULT_OPTIONS = {
  config: { attributes: true, childList: true, subtree: true },
};

export default function DontationPayment() {
  const [clientSecret, setClientSecret] = useState('');
  const [paymentIntent, setPaymentIntent] = useState('');

  const htmlEl = document.getElementsByTagName('html')[0];

  useEffect(() => {
    // Create PaymentIntent as soon as the page loads using our local API
    fetch('api/stripe_intent', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify({
        amount: 2000,
        payment_intent_id: '',
      }),
    })
      .then((res) => res.json())
      .then((data) => {
        setClientSecret(data.client_secret);
        setPaymentIntent(data.id);
      });
  }, []);

  const styles = getComputedStyle(htmlEl);

  const appearance: Appearance = {
    theme: 'stripe',
    variables: {
      colorBackground: styles.getPropertyValue('--stripe-background'),
      colorText: styles.getPropertyValue('--stripe-foreground'),
    },
    labels: 'floating',
  };

  const options: StripeElementsOptions = {
    clientSecret,
    appearance,
  };

  return (
    <>
      {clientSecret && (
        <Elements options={options} stripe={stripe}>
          <CheckoutForm paymentIntentID={paymentIntent} />
        </Elements>
      )}
    </>
  );
}
